/**
 * Created by Gilad Glasberg on 18/03/14.
 *
 * Pretty much totally re-written by Mitchell McCann on 23/11/14.
 * He Pretty much rewrote the game himself.
 */

var player = new Player();	// See player.js
var computer = new Player();// See player.js
var game = new Game();		// See game.js

// Hides all of the cards and unnecessary buttons from the game screen.
function clearTheCardsAndButtons () {

	// The river cards
	$("#slot01img").hide();
	$("#slot02img").hide();
	$("#slot03img").hide();
	$("#slot04img").hide();
	$("#slot05img").hide();

	// Player and computer cards
	$("#userCard01").src = "";
    $("#userCard02").src = "";
    $("#block01").src = "";
    $("#block02").src = "";
    $("#computerCard01").src = "";
    $("#computerCard02").src = "";
    
	$("#blockCards").show();
	
	// Player actions
	$("#userAction01").hide();
	$("#userAction02").hide();
	$("#userAction03").hide();
	$("#userAction04").hide();
	
	// Deal a new hand and start a new game button
	$("#startButton").hide();
}

clearTheCardsAndButtons();

// Set the text for most of the spans in the game info screen
function fixTheSpans () {
	$("#userWins").text(player.wins);
	$("#userFolds").text(player.folds);
	$("#userLosses").text(player.losses);
	$("#computerWins").text(computer.wins);
	$("#computerFolds").text(computer.folds);
	$("#computerLosses").text(computer.losses);
	$("#userPot").text(player.pot);
	$("#compPot").text(computer.pot);
	$("#handPot").text(game.currentPot);
	$("#handBet").text(game.currentBet);
	
	$("#neededToCall").text(game.currentBet-player.currentBet);
	
	if (game.lastBetPlayer == 0)
	{
		$("#turnIndicator").text("The user.");
	}
	else
	{
		$("#turnIndicator").text("The opponent.");
	}
	
	$("#anteChip").css('top', 15 + (70*game.anteChip) + '%');
	
}

// Function that deals a new hand and starts the new hand.
// TODO: Incorporate the use of the Ante chips that are in the game object.
function mixedTheDeck (){

	$("#startButton").val("Deal a new hand.");
	clearTheCardsAndButtons();

	computer.bestPossibleHand = null;
	player.bestPossibleHand = null;
	game.bestPossibleHand = null;

	fixTheSpans();

    // shuffle the deck of cards
    game.shuffle();

	//Used for advice
	player.potAtBeginOfHand = player.pot;
	//alert(player.potAtBeginOfHand);

    if ( game.anteChip == 0 )
	{
		game.currentBet = computer.bet(game.ante);
		game.currentPot += game.currentBet;
	}
	else
	{
		game.currentBet = player.bet(game.ante);
		game.currentPot += game.currentBet;
	}	
	
	//clean table from the 5 cards in case you start another game from the start
    $("#fiveCards img").attr("src", "");

    // animation of cards dealing, just an animation
    $("#GFX01").show(1);
    $("#GFX01").addClass("effect01");
    $("#GFX01").hide(500);

    setTimeout(now02,250);
    function now02(){
		$("#GFX02").show(1);
		$("#GFX02").addClass("effect02");
		$("#GFX02").hide(700);
    }

    setTimeout(now03,750);
    function now03(){
        $("#GFX03").show(1);
        $("#GFX03").addClass("effect03");
        $("#GFX03").hide(500);
    }

    setTimeout(now04,1000);
    function now04(){
        $("#GFX04").show(1);
        $("#GFX04").addClass("effect04");
        $("#GFX04").hide(700);
    }

	// User Hand
    setTimeout(startItUser, 1700);
    function startItUser(){
		player.addToHand(game.getCard());
		player.addToHand(game.getCard());
		
		document.getElementById("userCard01").src = player.hand[0].src;
		document.getElementById("userCard02").src = player.hand[1].src;
		$("#nameUser p").text("User");

        // Clearing the Animation
        $("#GFX01").removeClass("effect01");
        $("#GFX02").removeClass("effect02");
        $("#GFX03").removeClass("effect03");
        $("#GFX04").removeClass("effect04");
    }

    // Computer Hand
    setTimeout(startItComputer, 850);
    function startItComputer(){
	
		computer.addToHand(game.getCard());
		computer.addToHand(game.getCard());
		
		document.getElementById("computerCard01").src = computer.hand[0].src;
		document.getElementById("computerCard02").src = computer.hand[1].src;
		document.getElementById("block01").src = "img/cards/b1fv.png";
		document.getElementById("block02").src = "img/cards/b1fv.png";
		// Titles
		$("#nameComputer p").text("Computer");
		$("#userAction02").hide(1);
		$("#userAction03").hide(1);
		$("#userAction04").hide(1);
		$("#userAction01").show(1);
	}
	
	setTimeout(function(){
		$('#advice').val(determineAdvice(0));
	}, 2000);
	
	fixTheSpans();

	if ( game.anteChip == 1 )
	{
		setTimeout(function(){
			alert("The opponent starts this hand. You have ante'd in.");
			aiTurn();
		}, 2200);
	}
}

// The function that handles when the user calls the current game bet.
function userCall(){
	if ( (player.pot+player.currentBet) > (game.currentBet-player.currentBet) )
	{
		game.currentPot += player.bet(game.currentBet);
	}
	else	// All in
	{
		game.currentPot += parseInt(player.pot);
		player.pot = 0;
	}
	
	if (game.lastBetPlayer == 1)
	{		
		$("#userAction0" + game.currentState).hide(1);
		$("#userAction0" + ((game.currentState+1) % 5)).show(1);
		player.currentBet = 0;
		computer.currentBet = 0;
		dealCards();
	}
	else
	{
		aiTurn();
	}
	
	player.lastMove = "call";
}

// The function for the opponent to call the current bet. 
function compCall(){
	if ( computer.pot+computer.currentBet > game.currentBet )
	{		
		game.currentPot += computer.bet ( parseInt(game.currentBet) );
	}
	else	// All in
	{
		game.currentPot += computer.bet(computer.pot+computer.currentBet);
	}
	
	if ( game.currentBet == 0 )
	{
		alert ("The opponent has checked.");
	}
	else
	{
		alert ("The opponent has called.");
	}
		
	if (game.lastBetPlayer == 0)
	{
		dealCards();
		$("#userAction0" + game.currentState).hide(1);
		$("#userAction0" + ((game.currentState+1) % 5)).show(1);
		player.currentBet = 0;
		computer.currentBet = 0;
	}
	else
	{
		// human turn
	}
	
	computer.lastMove = "call";
}

// The function for the opponent to fold
function compFold()
{
	alert ("The opponent has folded.");
	clearTheCardsAndButtons();
	
	var cardsFromPlayer = player.win(game.currentPot);
	game.addCards(cardsFromPlayer);
	var cardsFromComputer = computer.fold(game.currentState);
	game.addCards(cardsFromComputer);
	game.reset();
	
	game.currentPot = 0;
	game.currentBet = 0;
	
    $("#advice").text("The AI has folded. A New Game Will Start Automatically").show(1).delay(4500).hide(1);
    setTimeout(mixedTheDeck, 4500);
	fixTheSpans();
}

// The function that will handle how the opponent plays.
function aiTurn() {	
	// This is where the logic for the opponent will go. 
	// Now, a Random Number Generator, that calls/raises randomly
	var rng = Math.random() * 11;
	if ( rng < 2 ) 
	{	// fold
		if ( (game.currentBet-computer.currentBet) == 0 )
		{
			compCall();
		} 
		else
		{
			compFold();
		}
	}
	else if ( rng < 7 )
	{	// raise
		var raiseAmount = Math.floor( Math.random() * Math.max( 200, computer.pot/5 ) );
		while (raiseAmount == 0)
		{
			raiseAmount = Math.floor( Math.random() * Math.max( 200, computer.pot/5 ) );
		}
		compRaise(raiseAmount);
			
	}
	else
	{	// call
		compCall();
	}
}

// Deal the cards. The number of cards dealt depends on the game state.
// Also resets the current game bet and launches the code to check for a winner.
// After all that is done, it returns the cards to the game.
// TODO: For some reason, it breaks after the second hand drawing. Never on the first. Only sometimes afterwards.
// TODO: After the ante chip is implemented, reset the game's last better every card deal.
//                        I think this might be implemented in the game.reset() method actually. Not sure
function dealCards() {
	player.lastMove = "";
	computer.lastMove = "";
	game.currentBet = 0;
	game.currentState = game.currentState+1;
	game.lastBetPlayer = game.anteChip;
	
	if ( game.lastBetPlayer == 1 && game.currentState != 4 )
	{
		aiTurn();
	}
	
	$("#userAction0" + (game.currentState-1)).hide();
	$("#userAction0" + game.currentState).show();
	
	if (game.currentState == 1)
	{
		flopTime();
	}
	else if (game.currentState == 2)
	{
		turnTime();
	}
	else if (game.currentState == 3)
	{
		riverTime();
	}
	else if (game.currentState == 4)
	{
		$("#blockCards").hide();
		$("#userAction01").hide();
		$("#userAction02").hide();
		$("#userAction03").hide();
		$("#userAction04").hide();
		
		var userHas = checkPossibleHands(player.hand, 0);
		var compHas = checkPossibleHands(computer.hand, 0);
		
		var playerHand;
		var compHand;
		
		// winner(); Currently uses the node program. I'm using my javascript checker.
		var whoWon = whoHasBetterHand ( userHas, compHas );
		if (whoWon < 0)	// Player won
		{
			playerHand = player.win(game.currentPot);
			compHand = computer.loseHand();
			alert ("User wins with a " + handTypeToString(userHas.handType));
		}
		else if (whoWon > 0) // Opponent won
		{
			playerHand = player.loseHand();
			compHand = computer.win(game.currentPot);
			alert ("Computer wins with a " + handTypeToString(compHas.handType));
		}
		else // Tie game
		{
			alert ("Tie game. Both players have a " + handTypeToString(userHas.handType));
			playerHand = player.tie(game.currentPot);
			compHand = computer.tie(game.currentPot);
		}
		
		game.addCards(playerHand);
		game.addCards(compHand);
		game.reset();
		game.currentState = 0;
		
		if ( computer.pot == 0 )
		{
			alert ("Congratulations! You win!");
		}
		else if ( player.pot == 0 )
		{
			alert ("Oh no! You've lost! Sad day. :(");
		}
		else
		{
			fixTheSpans();
			$("#advice").text("The hand has ended. A New Game Will Start Automatically").show(1).delay(4500).hide(1);
			setTimeout(mixedTheDeck, 4500);
			//$("#startButton").show();
		}
	}
	
	if (game.currentState != 4)
	{
		$('#advice').val(determineAdvice(game.currentState));
	}
	fixTheSpans();
}

// Launched by a button click. Prompts the user for amount to raise BY, not raise to.
// Changes the betting state so the game knows the user is the last person who raised.
// TODO: (optional feature) maybe add an "all in" indicator to just draw cards and skip betting in future betting rounds.
function userRaise(){
	var amount = parseInt(prompt("How much would you like to raise by?\nEnter 0 to only call.", 0));
	if ( amount+parseInt(game.currentBet) > (player.pot+player.currentBet) )	// Bet is more than player has.
	{
		if ( (player.pot+player.currentBet) > (computer.pot+computer.currentBet) )
		{
			alert ( "Current Proposed Bet = " + (amount + parseInt(game.currentBet)) + "\nThe opponent has insufficient funds. Will only raise to " + computer.pot);
			game.raiseTo(computer.pot+computer.currentBet);
		}
		else
		{
			alert ( "Current Proposed Bet = " + (amount + parseInt(game.currentBet)) + "\nInsufficient funds. Will only raise to " + player.pot);
			game.raiseTo(player.pot+player.currentBet);
		}
	}
	else if ( amount+parseInt(game.currentBet) > (computer.pot + computer.currentBet) ) // Bet is more than the opponent has
	{
		alert ( "Current Proposed Bet = " + (amount + parseInt(game.currentBet)) + "\nThe opponent has insufficient funds. Will only raise to " + computer.pot);
		game.raiseTo(computer.pot+computer.currentBet);
	}
	else // Its a valid bet
	{
		game.raiseBy(amount);
	}
	game.lastBetPlayer = 0;	
	player.bet(game.currentBet);
	game.currentPot += game.currentBet;
	player.lastMove = "raise";
	
	// TEMPORARY
	aiTurn();
	
	fixTheSpans();
}

// Makes a raise for the opponent in a given amount. Raises BY, not to, the amount given.
// Also, changes the betting state so the game knows the opponent is the last person who raised.
// TODO: (optional feature) maybe add an "all in" indicator to just draw cards and skip betting in future betting rounds.
function compRaise(amount){

	if ( player.pot == 0 || computer.pot == 0 )
	{
		// Don't bet. We have no money to bet.
		compCall();
	}
	else
	{
		var currentBet = game.currentBet;
		if ( amount+parseInt(game.currentBet) > (player.pot+player.currentBet) )	// Bet is more than player has.
		{
			if ( (player.pot+player.currentBet) > (computer.pot+computer.currentBet) )
			{
				game.raiseTo(computer.pot+computer.currentBet);	// Computer is now all in
			}
			else
			{
				game.raiseTo(player.pot+player.currentBet); // Computer has some chips. Forcing the player all in.
			}
		}
		else if ( amount+parseInt(game.currentBet) > (computer.pot+computer.currentBet) ) // Bet is more than the opponent has
		{
			game.raiseTo(computer.pot);	// Computer is now all in. Player has some chips, unless they were tied before.
		}
		else // Bet is a valid amount
		{
			game.raiseBy(amount);
		}

		alert ( "The opponent raised the bet from " + currentBet + " to " + game.currentBet );

		game.lastBetPlayer = 1;
		game.currentPot += computer.bet(game.currentBet);
		computer.lastMove = "raise";

		fixTheSpans();
	}
}

// Handles the case where the user folds. Launched by a button click.
// Clears the screen and prepares for the next hand. 
// Gives the opponent his winnings, and increments the proper stats.
// TODO: (optional) currently just starts a new hand after 4.5 seconds. Maybe wait for the button click instead?
function userFold(){
    clearTheCardsAndButtons();
	
	var cardsFromPlayer = player.fold(game.currentState);
	game.addCards(cardsFromPlayer);
	var cardsFromComputer = computer.win(game.currentPot);
	game.addCards(cardsFromComputer);
	game.reset();
	
	game.currentPot = 0;
	game.currentBet = 0;
	
    $("#advice").text("A New Game Will Start Automatically").show(1).delay(4500).hide(1);
    setTimeout(mixedTheDeck, 4500);
	fixTheSpans();
}

// Put 3 cards on the table, and prepare for the next round of betting.
// Currently gives the best possible hand type the opponent could have, 
// as well as the hand that the player has.
function flopTime (){
	//first 3 cards
	game.river[0] = game.getCard();
	game.river[1] = game.getCard();
	game.river[2] = game.getCard();
	
	$("#slot01img").hide(1);
	$("#slot01img").attr("src", game.river[0].src).show(500);

	$("#slot02img").hide(1);
	$("#slot02img").attr("src", game.river[1].src).delay(500).show(500);

	$("#slot03img").hide(1);
	$("#slot03img").attr("src", game.river[2].src).delay(1000).show(500);
	
	// Returns an array of card values and suits. i.e., [[1,'h'],[2,'c'],[3,'s'],[5,'d'],[6,'h']]
	//game.worstPossibleHand = getWorstPossibleHandFlopOnly();
	game.bestPossibleHand = check3cards();
	// bestCompHand[0] = hand
	// bestCompHand[1] = hand
	//		hand = [[card1 value, card1 suit], [card2 value, card2 suit], [card3 value, card3 suit]
	//				[card4 value, card4 suit], [card5 value, card5 suit]]
	$("#compHand").text(game.bestPossibleHand[0]);
	
	// Returns an object with the player cards now. The cards are part of the object.
	//         i.e., a card is: player.bestPossibleHand.card1.val + player.bestPossibleHand.card1.myType
	//         Cards are numbered card1 through card5. Skipping 0 index.
	player.bestPossibleHand = checkPossibleHandsFlopOnly(player.hand, 0);
	$("#userHas").text(handTypeToString(player.bestPossibleHand.handType));
	
}

// Puts the next card on the table and prepares for the next round of betting.
// Currently prints the best possible hand type the opponent could have,
// as well as the hand that the play does have.
function turnTime (){
	game.river[3] = game.getCard();
	
	$("#slot04img").hide(1);
	$("#slot04img").attr("src", game.river[3].src).show(500);
	
	// Returns an array of card values and suits. i.e., [[1,'h'],[2,'c'],[3,'s'],[5,'d'],[6,'h']]
	//game.getWorstPossibleHandTurnOnly();
	game.bestPossibleHand = check4cards();
	// bestCompHand[0] = hand
	// bestCompHand[1] = hand
	//		hand = [[card1 value, card1 suit], [card2 value, card2 suit], [card3 value, card3 suit]
	//				[card4 value, card4 suit], [card5 value, card5 suit]]
	$("#compHand").text(game.bestPossibleHand[0]);
	
	// Returns an object with the player cards now. The cards are part of the object.
	//         i.e., a card is: player.bestPossibleHand.card1.val + player.bestPossibleHand.card1.myType
	//         Cards are numbered card1 through card5. Skipping 0 index.
	player.bestPossibleHand = checkPossibleHandsNoRiver(player.hand, 0);
	$("#userHas").text(handTypeToString(player.bestPossibleHand.handType));
}

// Puts the next card on the table and prepares for the next round of betting.
// Currently prints the best possible hand type the opponent could have,
// as well as the hand that the play does have.
function riverTime (){
	game.river[4] = game.getCard();
	
	$("#slot05img").hide(1);
	$("#slot05img").attr("src", game.river[4].src).show(500);
	
	// Returns an array of card values and suits. i.e., [[1,'h'],[2,'c'],[3,'s'],[5,'d'],[6,'h']]
	//game.worstPossibleHand = getWorstPossibleHandRiver(); // This is really just the common cards sorted and turned into the same object style.
	game.bestPossibleHand = check5cards();
	// bestCompHand[0] = hand
	// bestCompHand[1] = hand
	//		hand = [[card1 value, card1 suit], [card2 value, card2 suit], [card3 value, card3 suit]
	//				[card4 value, card4 suit], [card5 value, card5 suit]]
	$("#compHand").text(game.bestPossibleHand[0]);
	
	// Returns an object with the player cards now. The cards are part of the object.
	//         i.e., a card is: player.bestPossibleHand.card1.val + player.bestPossibleHand.card1.myType
	//         Cards are numbered card1 through card5. Skipping 0 index.
	player.bestPossibleHand = checkPossibleHands(player.hand, 0);
	$("#userHas").text(handTypeToString(player.bestPossibleHand.handType));
	
}

// Other tasks to do:




















